﻿using System;
using System.Threading;
using System.Threading.Tasks;
using Microsoft.Phone.Net.NetworkInformation;

namespace BC3Technologies.Common.Mobile
{
	public class NetworkTypeEventArgs
	{
		public NetworkTypeEventArgs(NetworkInterfaceType type, bool hasTimeout = false)
		{
			Type = type;
			HasTimeout = hasTimeout;
		}

		public bool HasTimeout { get; private set; }

		public bool HasInternet
		{
			get { return Type != NetworkInterfaceType.None; }
		}

		public NetworkInterfaceType Type { get; private set; }
	}

	/// <summary> 
	/// Static class to get the NetworkInterfaceType without blocking the UI thread.
	/// </summary> 
	public class NetworkInformationUtility
	{
		private static bool _isGettingNetworkType;
		private static readonly object _synchronizationObject = new object();
		private static Timer _timer;

		/// <summary> 
		/// Get the NetworkInterfaceType asynchronously. 
		/// </summary>
		/// /// <param name="timeoutInMs">Specifies the timeout in milliseconds.</param> 
		private void GetNetworkTypeAsync(int timeoutInMs)
		{
			lock (_synchronizationObject)
			{
				if (!_isGettingNetworkType)
				{
					_isGettingNetworkType = true;

					if (System.Net.NetworkInformation.NetworkInterface.GetIsNetworkAvailable())
					{
						Thread thread = new Thread(GetNetworkType) { IsBackground = true };
						thread.Start(timeoutInMs);
					}
					else
					{
						FireGetNetworkTypeCompleted(NetworkInterfaceType.None);
					}
				}
			}
		}

		public static Task<NetworkTypeEventArgs> GetNetworkTypeTaskAsync(int timeoutInMs)
		{
			var tcs = new TaskCompletionSource<NetworkTypeEventArgs>();

			try
			{
				var infoFetcher = new NetworkInformationUtility();
				infoFetcher.GetNetworkTypeCompleted += (s, e) =>
				{
					tcs.TrySetResult(e);
				};

				infoFetcher.GetNetworkTypeAsync(timeoutInMs);
			}
			catch (Exception ex)
			{
				tcs.TrySetException(ex);
			}

			if (tcs.Task.Exception != null)
			{
				throw tcs.Task.Exception;
			}

			return tcs.Task;
		}

		public delegate void NetworkTypeEventHandler(object sender, NetworkTypeEventArgs networkTypeEventArgs);

		public event NetworkTypeEventHandler GetNetworkTypeCompleted;

		private void OnTimerElapsed(object state)
		{
			FireGetNetworkTypeCompleted(NetworkInterfaceType.None, true);
		}

		private void GetNetworkType(object state)
		{
			using (_timer = new Timer(OnTimerElapsed, null, (int)state, 0))
			{
				// This is a blocking call, this is why a thread is used to let the UI to be fluid 
				NetworkInterfaceType type = NetworkInterface.NetworkInterfaceType;
				FireGetNetworkTypeCompleted(type);
			}
		}

		private void FireGetNetworkTypeCompleted(NetworkInterfaceType type, bool hasTimeout = false)
		{
			lock (_synchronizationObject)
			{
				if (_isGettingNetworkType)
				{
					_isGettingNetworkType = false;

					NetworkTypeEventHandler networkTypeEventHandler = GetNetworkTypeCompleted;

					if (networkTypeEventHandler != null)
					{
						networkTypeEventHandler(null, new NetworkTypeEventArgs(type, hasTimeout));
					}
				}
			}
		}
	}
}